@@ -85,18 +85,31 @@ const GList * gdm_peek_local_address_list (void);
 gboolean gdm_is_local_addr (struct in_addr *ia);
 gboolean gdm_is_loopback_addr (struct in_addr *ia);
 
+#ifdef ENABLE_IPV6
+gboolean gdm_is_local_addr6 (struct in6_addr* ia);
+gboolean gdm_is_loopback_addr6 (struct in6_addr *ia);
+#endif
+
 typedef struct {
 	gboolean not_found; /* hostname below set to fallback,
 			       as gethostbyaddr/name failed */
 	char *hostname; /* never a bogus dot, if
 			   invalid/unknown, then set to the
 			   ip address in string form */
+#ifdef ENABLE_IPV6
+	struct sockaddr_storage *addrs;
+#else
 	struct in_addr *addrs; /* array */
+#endif
 	int addr_count;
 } GdmHostent;
 
 GdmHostent * gdm_gethostbyname (const char *name);
-GdmHostent * gdm_gethostbyaddr (struct in_addr *ia);
+#ifdef ENABLE_IPV6
+GdmHostent *gdm_gethostbyaddr (struct sockaddr_storage *ia);
+#else
+GdmHostent * gdm_gethostbyaddr (struct sockaddr_in *ia);
+#endif
 GdmHostent * gdm_hostent_copy (GdmHostent *he);
 void gdm_hostent_free (GdmHostent *he);
 
@@ -116,6 +129,7 @@ char * gdm_locale_to_utf8 (const char *text);
 char * gdm_locale_from_utf8 (const char *text);
 
 void gdm_saveenv (void);
+const char * gdm_saved_getenv (const char *var);
 /* leaks */
 void gdm_restoreenv (void);
 
@@ -156,6 +170,17 @@ char * gdm_make_filename (const char *dir, const char *name, const char *extensi
 char * gdm_ensure_extension (const char *name, const char *extension);
 char * gdm_strip_extension (const char *name, const char *extension);
 
+void gdm_twiddle_pointer (GdmDisplay *disp);
+
+char * gdm_get_last_info (const char *username);
+
+gboolean gdm_ok_console_language (void);
+const char * gdm_console_translate (const char *str);
+/* Use with C_(N_("foo")) to make gettext work it out right */
+#define C_(x) (gdm_console_translate(x))
+
+gchar * gdm_read_default (gchar *key);
+
 #endif /* GDM_MISC_H */
 
 /* EOF */
