@@ -17,20 +17,19 @@
  */
 
 #include <config.h>
-#include <libgnome/libgnome.h>
-#include <gtk/gtkmessagedialog.h>
+
 #include <grp.h>
-#include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <syslog.h>
 #include <security/pam_appl.h>
 #include <pwd.h>
-
-#ifdef HAVE_DEFOPEN
-#include <deflt.h>
+#ifdef sun
+#include <fcntl.h>
 #endif
 
+#include <libgnome/libgnome.h>
 #include <vicious.h>
 
 #include "gdm.h"
@@ -57,6 +56,7 @@ extern gint GdmRetryDelay;
 extern gboolean GdmDisplayLastLogin;
 extern uid_t GdmUserId;
 extern gid_t GdmGroupId;
+extern gboolean GdmPasswordRequired;
 
 extern gboolean GdmConsoleNotify;
 
@@ -100,47 +100,48 @@ static	adt_session_data_t      *adt_ah = NULL;    /* audit session handle */
  *	
  */
 static void
-audit_success_login(int pw_change, struct passwd *pwent)
+audit_success_login (int pw_change, struct passwd *pwent)
 {
 	adt_event_data_t	*event;	/* event to generate */
 
-	if (adt_start_session(&adt_ah, NULL, ADT_USE_PROC_DATA) != 0) {
+	if (adt_start_session (&adt_ah, NULL, ADT_USE_PROC_DATA) != 0) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_start_session(ADT_login): %m");
 		return;
 	}
-	if (adt_set_user(adt_ah, pwent->pw_uid, pwent->pw_gid,
+
+	if (adt_set_user (adt_ah, pwent->pw_uid, pwent->pw_gid,
 	    pwent->pw_uid, pwent->pw_gid, NULL, ADT_USER) != 0) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_set_user(ADT_login, %s): %m", pwent->pw_name);
 	}
-	if ((event = adt_alloc_event(adt_ah, ADT_login)) == NULL) {
+	if ((event = adt_alloc_event (adt_ah, ADT_login)) == NULL) {
 
-		syslog(LOG_AUTH | LOG_ALERT, "adt_alloc_event(ADT_login): %m");
+		syslog (LOG_AUTH | LOG_ALERT, "adt_alloc_event(ADT_login): %m");
 	} else if (adt_put_event(event, ADT_SUCCESS, ADT_SUCCESS) != 0) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_put_event(ADT_login, ADT_SUCCESS): %m");
 	}
 
 	if (pw_change == PW_TRUE) {
 
 		/* Also audit password change */
-		adt_free_event(event);
-		if ((event = adt_alloc_event(adt_ah, ADT_passwd)) == NULL) {
+		adt_free_event (event);
+		if ((event = adt_alloc_event (adt_ah, ADT_passwd)) == NULL) {
 
-			syslog(LOG_AUTH | LOG_ALERT,
+			syslog (LOG_AUTH | LOG_ALERT,
 			    "adt_alloc_event(ADT_passwd): %m");
-		} else if (adt_put_event(event, ADT_SUCCESS,
+		} else if (adt_put_event (event, ADT_SUCCESS,
 		    ADT_SUCCESS) != 0) {
 
-			syslog(LOG_AUTH | LOG_ALERT,
-			    "adt_put_event(ADT_passwd, ADT_SUCCESS): %m");
+			syslog (LOG_AUTH | LOG_ALERT,
+			    "adt_put_event (ADT_passwd, ADT_SUCCESS): %m");
 		}
 	}
-	adt_free_event(event);
+	adt_free_event (event);
 }
 
 
@@ -165,7 +166,7 @@ audit_success_login(int pw_change, struct passwd *pwent)
  *	
  */
 static void
-audit_fail_login(GdmDisplay *d, int pw_change, struct passwd *pwent,
+audit_fail_login (GdmDisplay *d, int pw_change, struct passwd *pwent,
     int pamerr)
 {
 	adt_session_data_t	*ah;	/* audit session handle */
@@ -173,14 +174,14 @@ audit_fail_login(GdmDisplay *d, int pw_change, struct passwd *pwent,
 	adt_termid_t		*tid;	/* terminal ID for failures */
 
 	if (did_setcred == TRUE) {
-		if (adt_start_session(&ah, NULL, ADT_USE_PROC_DATA) != 0) {
+		if (adt_start_session (&ah, NULL, ADT_USE_PROC_DATA) != 0) {
 
 			syslog(LOG_AUTH | LOG_ALERT,
 			    "adt_start_session(ADT_login, ADT_FAILURE): %m");
 			return;
 		}
 	} else {
-		if (adt_start_session(&ah, NULL, 0) != 0) {
+		if (adt_start_session (&ah, NULL, 0) != 0) {
 		
 			syslog(LOG_AUTH | LOG_ALERT,
 			    "adt_start_session(ADT_login, ADT_FAILURE): %m");
@@ -188,84 +189,84 @@ audit_fail_login(GdmDisplay *d, int pw_change, struct passwd *pwent,
 		}
 		if (d->attached) {
 			/* login from the local host */
-			if (adt_load_hostname(NULL, &tid) != 0) {
+			if (adt_load_ttyname ("/dev/console", &tid) != 0) {
 
-				syslog(LOG_AUTH | LOG_ALERT,
+				syslog (LOG_AUTH | LOG_ALERT,
 				    "adt_loadhostname(localhost): %m");
 			}
 		} else {
 			/* login from a remote host */
-			if (adt_load_hostname(d->hostname, &tid) != 0) {
+			if (adt_load_hostname (d->hostname, &tid) != 0) {
 
-				syslog(LOG_AUTH | LOG_ALERT,
+				syslog (LOG_AUTH | LOG_ALERT,
 				    "adt_loadhostname(%s): %m", d->hostname);
 			}
 		}
-		if (adt_set_user(ah,
+		if (adt_set_user (ah,
 		    pwent ? pwent->pw_uid : ADT_NO_ATTRIB,
 		    pwent ? pwent->pw_gid : ADT_NO_ATTRIB,
 		    pwent ? pwent->pw_uid : ADT_NO_ATTRIB,
 		    pwent ? pwent->pw_gid : ADT_NO_ATTRIB,
 		    tid, ADT_NEW) != 0) {
 
-			syslog(LOG_AUTH | LOG_ALERT,
+			syslog (LOG_AUTH | LOG_ALERT,
 			    "adt_set_user(%s): %m",
 			    pwent ? pwent->pw_name : "ADT_NO_ATTRIB");
 		}
 	}
-	if ((event = adt_alloc_event(ah, ADT_login)) == NULL) {
+	if ((event = adt_alloc_event (ah, ADT_login)) == NULL) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
-		    "adt_alloc_event(ADT_login, ADT_FAILURE): %m");
+		syslog (LOG_AUTH | LOG_ALERT,
+		    "adt_alloc_event (ADT_login, ADT_FAILURE): %m");
 		goto done;
 	} else if (adt_put_event(event, ADT_FAILURE,
 	    ADT_FAIL_PAM + pamerr) != 0) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_put_event(ADT_login(ADT_FAIL, %s): %m",
-		    pam_strerror(pamh, pamerr));
+		    pam_strerror (pamh, pamerr));
 	}
 	if (pw_change != PW_FALSE) {
 
 		/* Also audit password change */
-		adt_free_event(event);
-		if ((event = adt_alloc_event(ah, ADT_passwd)) == NULL) {
+		adt_free_event (event);
+		if ((event = adt_alloc_event (ah, ADT_passwd)) == NULL) {
 
-			syslog(LOG_AUTH | LOG_ALERT,
+			syslog (LOG_AUTH | LOG_ALERT,
 			    "adt_alloc_event(ADT_passwd): %m");
 			goto done;
 		}
 		if (pw_change == PW_TRUE) {
-			if (adt_put_event(event, ADT_SUCCESS,
+			if (adt_put_event (event, ADT_SUCCESS,
 			    ADT_SUCCESS) != 0) {
 
-				syslog(LOG_AUTH | LOG_ALERT,
+				syslog (LOG_AUTH | LOG_ALERT,
 				    "adt_put_event(ADT_passwd, ADT_SUCCESS): "
 				    "%m");
 			}
 		} else if (pw_change == PW_FAILED) {
-			if (adt_put_event(event, ADT_FAILURE,
+			if (adt_put_event (event, ADT_FAILURE,
 			    ADT_FAIL_PAM + pamerr) != 0) {
 
-				syslog(LOG_AUTH | LOG_ALERT,
+				syslog (LOG_AUTH | LOG_ALERT,
 				    "adt_put_event(ADT_passwd, ADT_FAILURE): "
 				    "%m");
 			}
 		}
 	}
-	adt_free_event(event);
+	adt_free_event (event);
 
 done:
 	/* reset process audit state. this process is being reused.*/
 
-	if ((adt_set_user(ah, ADT_NO_AUDIT, ADT_NO_AUDIT, ADT_NO_AUDIT,
+	if ((adt_set_user (ah, ADT_NO_AUDIT, ADT_NO_AUDIT, ADT_NO_AUDIT,
 	    ADT_NO_AUDIT, NULL, ADT_NEW) != 0) ||
-	    (adt_set_proc(ah) != 0)) {
+	    (adt_set_proc (ah) != 0)) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_put_event(ADT_login(ADT_FAILURE reset, %m)");
 	}
-	(void) adt_end_session(ah);
+	(void) adt_end_session (ah);
 }
 
 
@@ -273,7 +274,7 @@ done:
  * audit_logout - audit user logout
  *
  *	Entry	adt_ah = audit session handle established by
- *			audit_success_login().
+ *			 audit_success_login().
  *
  *	Exit	ADT_logout (ADT_SUCCESS) audit record written
  *		process audit state reset.  (this process is reused for
@@ -281,17 +282,11 @@ done:
  *		audit session adt_ah ended.
  */
 static void
-audit_logout(void)
+audit_logout (void)
 {
 	adt_event_data_t	*event;	/* event to generate */
 
-	if (adt_ah == NULL) {
-
-		syslog(LOG_AUTH | LOG_ALERT,
-		    "adt_ah(ADT_logout): NULL");
-		return;
-	}
-	if ((event = adt_alloc_event(adt_ah, ADT_logout)) == NULL) {
+	if ((event = adt_alloc_event (adt_ah, ADT_logout)) == NULL) {
 
 		syslog(LOG_AUTH | LOG_ALERT,
 		    "adt_alloc_event(ADT_logout): %m");
@@ -304,17 +299,119 @@ audit_logout(void)
 
 	/* reset process audit state. this process is being reused.*/
 
-	if ((adt_set_user(adt_ah, ADT_NO_AUDIT, ADT_NO_AUDIT, ADT_NO_AUDIT,
+	if ((adt_set_user (adt_ah, ADT_NO_AUDIT, ADT_NO_AUDIT, ADT_NO_AUDIT,
 	    ADT_NO_AUDIT, NULL, ADT_NEW) != 0) ||
-	    (adt_set_proc(adt_ah) != 0)) {
+	    (adt_set_proc (adt_ah) != 0)) {
 
-		syslog(LOG_AUTH | LOG_ALERT,
+		syslog (LOG_AUTH | LOG_ALERT,
 		    "adt_set_proc(ADT_logout reset): %m");
 	}
-	(void) adt_end_session(adt_ah);
+	(void) adt_end_session (adt_ah);
 }
 #endif	/* HAVE_ADT */
 
+#ifdef sun
+void
+solaris_xserver_cred (char *login, GdmDisplay *d, struct passwd *pwent)
+{
+    	struct stat statbuf;
+        struct group *gr;
+	gid_t  groups[NGROUPS_UMAX];
+	char *home, *disp, *tmp, pipe[MAXPATHLEN], info[MAXPATHLEN];
+	int displayNumber = 0;
+	int retval, fd, i, nb;
+	int ngroups;
+
+	if (!d->attached)
+	    return;
+
+	if (access (pwent->pw_dir, F_OK) != 0) {
+	    gdm_debug ("solaris_xserver_cred: no HOME dir access\n");
+	    return;
+	}
+
+	/*
+	 * Handshake with server. Make sure it created a pipe.
+	 * Open and write.
+	 */
+        if ((tmp = strstr (d->name, ":")) != NULL) {
+		tmp++;
+                displayNumber = atoi (tmp);
+	}
+
+        sprintf (pipe, "%s/%d", SDTLOGIN_DIR, displayNumber);
+
+        if ( stat (SDTLOGIN_DIR, &statbuf) == 0) {
+	    if (! statbuf.st_mode & S_IFDIR) {
+		gdm_debug ("solaris_xserver_cred: %s is not a directory\n",
+		       SDTLOGIN_DIR);
+		return;
+	    }
+	}
+	else {
+            gdm_debug ("solaris_xserver_cred: %s does not exist\n", SDTLOGIN_DIR);
+	    return;
+	}
+
+	fd = open (pipe, O_RDWR);
+	unlink (pipe);
+
+	if (fd < 0) {
+            gdm_debug ("solaris_xserver_cred: could not open %s\n", pipe);
+	    return;
+	}
+        if ( fstat(fd, &statbuf) == 0 ) {
+	   if ( ! statbuf.st_mode & S_IFIFO) {
+	      close (fd);
+	      gdm_debug ("solaris_xserver_cred: %s is not a pipe\n", pipe);
+	      return;
+	   }
+	} else {
+	    close (fd);
+            gdm_debug ("solaris_xserver_cred: %s does not exist\n", pipe);
+	    return;
+	}
+	 
+	sprintf (info, "GID=\"%d\"; ", pwent->pw_gid);
+	nb = write (fd, info, strlen (info));
+        gdm_debug ("solaris_xserver_cred: %s\n", info);
+
+	if (initgroups (login, pwent->pw_gid) == -1) {
+	    ngroups = 0;
+	} else {
+	    ngroups = getgroups( NGROUPS_UMAX, groups);
+	}
+
+        for (i=0; i < ngroups; i++) {
+            sprintf (info, "G_LIST_ID=\"%u\" ", groups[i]);
+	    nb = write (fd, info, strlen(info));
+            gdm_debug ("solaris_xserver_cred: %s\n", info);
+	}
+
+	if (ngroups > 0) {
+            sprintf (info, ";");
+	    write (fd, info, strlen(info));
+	}
+	
+        sprintf (info, " HOME=\"%s\" ", pwent->pw_dir);
+	nb = write (fd, info, strlen(info));
+        gdm_debug ("solaris_xserver_cred: %s\n", info);
+
+        sprintf (info, " UID=\"%d\" EOF=\"\";", pwent->pw_uid);
+	nb = write (fd, info, strlen(info));
+        gdm_debug ("solaris_xserver_cred: %s\n", info);
+
+	/*
+	 * Handshake with server. Make sure it read the pipe.
+	 * 
+	 * Close file descriptor. 
+	 */
+ 	close (fd);
+	 
+	return;
+}
+#endif
+
 void
 gdm_verify_select_user (const char *user)
 {
@@ -386,17 +483,17 @@ gdm_verify_pam_conv (int num_msg, const struct pam_message **msg,
     if (pamh == NULL)
 	return PAM_CONV_ERR;
     
-    reply = malloc (sizeof (struct pam_response) * num_msg);
-    
-    if (reply == NULL)
-	return PAM_CONV_ERR;
-
     /* Should never happen unless PAM is on crack and keeps asking questions
        after we told it to go away.  So tell it to go away again and
        maybe it will listen */
     if ( ! gdm_slave_action_pending ())
         return PAM_CONV_ERR;
 
+    reply = malloc (sizeof (struct pam_response) * num_msg);
+    
+    if (reply == NULL)
+	return PAM_CONV_ERR;
+
     memset (reply, 0, sizeof (struct pam_response) * num_msg);
 
     /* Here we set the login if it wasn't already set,
@@ -718,14 +815,15 @@ gdm_verify_user (GdmDisplay *d,
 		 const gchar *display,
 		 gboolean local) 
 {
-    gint pamerr;
-    const void *p;
-    char *login;
+    gint pamerr = 0;
     struct passwd *pwent = NULL;
-    gboolean error_msg_given;
-    gboolean credentials_set;
-    gboolean started_timer;
-    int null_tok;
+    const void *p;
+    char *login, *passreq, *consoleonly;
+    int null_tok = 0;
+    gboolean credentials_set = FALSE;
+    gboolean error_msg_given = FALSE;
+    gboolean started_timer   = FALSE;
+
 #ifdef HAVE_ADT
     int pw_change = PW_FALSE;   /* if got to trying to change password */
 #endif	/* HAVE_ADT */
@@ -753,24 +851,6 @@ verify_user_again:
 
     cur_gdm_disp = d;
 
-    /* A Solaris thing: */
-#ifdef HAVE_DEFOPEN
-    if (defopen(DEFLT"/login") == 0) {
-	    char *passreq;
-	    int def_flgs;
-
-	    def_flgs = defcntl(DC_GETFLAGS, 0);
-	    TURNOFF(def_flgs, DC_CASE);
-	    (void) defcntl (DC_SETFLAGS, def_flgs);	 /* ignore case */
-
-	    if (((passreq = defread("PASSREQ=")) != NULL) &&
-		g_ascii_strcasecmp (passreq, "YES") == 0)
-		    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
-
-	    (void) defopen(NULL); /* close defaults file  */
-    }
-#endif
-	    
 authenticate_again:
 
     /* hack */
@@ -795,10 +875,18 @@ authenticate_again:
 #endif /* PAM_FAIL_DELAY */
 #endif
 
-    did_we_ask_for_password = FALSE;
+    passreq = gdm_read_default ("PASSREQ=");
+    if ((passreq != NULL) &&
+	g_ascii_strcasecmp (passreq, "YES") == 0)
+	    GdmPasswordRequired = TRUE;
 
+    if (GdmPasswordRequired)
+	    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
+	    
     gdm_verify_select_user (NULL);
+
     /* Start authentication session */
+    did_we_ask_for_password = FALSE;
     if ((pamerr = pam_authenticate (pamh, null_tok)) != PAM_SUCCESS) {
 	    if ( ! ve_string_empty (selected_user)) {
 		    pam_handle_t *tmp_pamh;
@@ -858,6 +946,7 @@ authenticate_again:
 		    gdm_error (_("Couldn't authenticate user"));
 	    goto pamerr;
     }
+
     login = g_strdup ((const char *)p);
     /* kind of anal, the greeter likely already knows, but it could have
        been changed */
@@ -875,6 +964,12 @@ authenticate_again:
 	    goto verify_user_again;
     }
 
+    /* Check if user is root and is allowed to log in */
+    consoleonly = gdm_read_default ("CONSOLE=");
+    if ((consoleonly != NULL) &&
+	g_ascii_strcasecmp (consoleonly, "/dev/console") == 0)
+	    GdmAllowRemoteRoot = FALSE;
+
     pwent = getpwnam (login);
     if ( ( ! GdmAllowRoot ||
 	  ( ! GdmAllowRemoteRoot && ! local) ) &&
@@ -890,12 +985,12 @@ authenticate_again:
 	      _("Root login disallowed"));*/
 	    error_msg_given = TRUE;
 
-#ifdef HAVE_ADT
-	/*
-	 * map console login not allowed as a pam_acct_mgmt() failure
-	 * indeed that's where these checks should be made.
-	 */
-    pamerr = PAM_PERM_DENIED;
+#ifdef  HAVE_ADT
+	   /*
+	    * map console login not allowed as a pam_acct_mgmt() failure
+	    * indeed that's where these checks should be made.
+	    */
+	    pamerr = PAM_PERM_DENIED;
 #endif	/* HAVE_ADT */
 	    goto pamerr;
     }
@@ -913,14 +1008,14 @@ authenticate_again:
 		      "Please try again later or contact the system administrator."));
 	    error_msg_given = TRUE;
 #ifdef  HAVE_ADT
-		/* Password change failed */
-		pw_change = PW_FAILED;
+	    /* Password change failed */
+	    pw_change = PW_FAILED;
 #endif	/* HAVE_ADT */
 	    goto pamerr;
 	}
 #ifdef  HAVE_ADT
-		/* Password changed */
-		pw_change = PW_TRUE;
+	/* Password changed */
+	pw_change = PW_TRUE;
 #endif	/* HAVE_ADT */
         break;
     case PAM_ACCT_EXPIRED :
@@ -950,16 +1045,21 @@ authenticate_again:
 					    "you will not be able to log in. "
 					    "Please contact your system administrator."));
 #ifdef  HAVE_ADT
-		/*
-		 * map group setup error as a pam_setcred() failure
-		 * indeed that's where this should be done.
-		 */
-		pamerr = PAM_SYSTEM_ERR;
+	    /*
+	     * map group setup error as a pam_setcred() failure
+	     * indeed that's where this should be done.
+	     */
+	    pamerr = PAM_SYSTEM_ERR;
 #endif	/* HAVE_ADT */
 	    goto pamerr;
     }
 
     did_setcred = TRUE;
+
+#ifdef sun
+    solaris_xserver_cred (login, d, pwent);
+#endif
+
     /* Set credentials */
     pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED);
     if (pamerr != PAM_SUCCESS) {
@@ -970,8 +1070,8 @@ authenticate_again:
     }
 
     credentials_set = TRUE;
+    opened_session  = TRUE;
 
-    opened_session = TRUE;
     /* Register the session */
     pamerr = pam_open_session (pamh, 0);
     if (pamerr != PAM_SUCCESS) {
@@ -993,19 +1093,19 @@ authenticate_again:
     tmp_PAM_USER = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->attached)
-		(void) di_devperm_login("/dev/console", pwent->pw_uid,
-		    pwent->pw_gid, NULL);
+    if (d->attached)
+	(void) di_devperm_login ("/dev/console", pwent->pw_uid,
+	    pwent->pw_gid, NULL);
 #endif	/* HAVE_LOGINDEVPERM */
 #ifdef  HAVE_ADT
-	audit_success_login(pw_change, pwent);
+    audit_success_login (pw_change, pwent);
 #endif  /* HAVE_ADT */
 
     return login;
     
  pamerr:
 #ifdef  HAVE_ADT
-	audit_fail_login(d, pw_change, pwent, pamerr);
+    audit_fail_login (d, pw_change, pwent, pamerr);
 #endif	/* HAVE_ADT */
 
     /* The verbose authentication is turned on, output the error
@@ -1102,9 +1202,13 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     gint pamerr = 0;
     struct passwd *pwent = NULL;
     const void *p;
-    const char *after_login;
+    char *passreq;
     int null_tok = 0;
+    gboolean credentials_set;
+    const char *after_login;
     
+    credentials_set = FALSE;
+
 #ifdef HAVE_ADT
     int pw_change = PW_FALSE;   /* if got to trying to change password */
 #endif	/* HAVE_ADT */
@@ -1127,23 +1231,13 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	    goto setup_pamerr;
     }
 
-    /* A Solaris thing: */
-#ifdef HAVE_DEFOPEN
-    if (defopen(DEFLT"/login") == 0) {
-	    char *passreq;
-	    int def_flgs;
+    passreq = gdm_read_default ("PASSREQ=");
+    if ((passreq != NULL) &&
+	g_ascii_strcasecmp (passreq, "YES") == 0)
+	    GdmPasswordRequired = TRUE;
 
-	    def_flgs = defcntl(DC_GETFLAGS, 0);
-	    TURNOFF(def_flgs, DC_CASE);
-	    (void) defcntl (DC_SETFLAGS, def_flgs);	 /* ignore case */
-
-	    if (((passreq = defread("PASSREQ=")) != NULL) &&
-		g_ascii_strcasecmp (passreq, "YES") == 0)
-		    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
-
-	    (void) defopen(NULL); /* close defaults file  */
-    }
-#endif
+    if (GdmPasswordRequired)
+	    null_tok |= PAM_DISALLOW_NULL_AUTHTOK;
 
     /* Start authentication session */
     did_we_ask_for_password = FALSE;
@@ -1175,8 +1269,8 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     }
 
 #ifdef  HAVE_ADT
-	/* to set up for same auditing calls as in gdm_verify_user */
-	pwent = getpwnam (login);
+    /* to set up for same auditing calls as in gdm_verify_user */
+    pwent = getpwnam (login);
 #endif	/* HAVE_ADT */
 
     /* Check if the user's account is healthy. */
@@ -1197,12 +1291,12 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 		    _("\nThe change of the authentication token failed. "
 		      "Please try again later or contact the system administrator."));
 #ifdef  HAVE_ADT
-		pw_change = PW_FAILED;
+	    pw_change = PW_FAILED;
 #endif	/* HAVE_ADT */
 	    goto setup_pamerr;
 	}
 #ifdef  HAVE_ADT
-    pw_change = PW_TRUE;
+	pw_change = PW_TRUE;
 #endif	/* HAVE_ADT */
 #endif	/* 0 */
         break;
@@ -1234,17 +1328,21 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 			     "you will not be able to log in. "
 			     "Please contact your system administrator."));
 #ifdef  HAVE_ADT
-		/*
-		 * map group setup error as a pam_setcred() failure
-		 * indeed that's where this should be done.
-		 */
-		pamerr = PAM_SYSTEM_ERR;
+	    /*
+	     * map group setup error as a pam_setcred() failure
+	     * indeed that's where this should be done.
+	     */
+	    pamerr = PAM_SYSTEM_ERR;
 #endif	/* HAVE_ADT */
 	    goto setup_pamerr;
     }
 
     did_setcred = TRUE;
 
+#ifdef sun
+    solaris_xserver_cred (login, d, pwent);
+#endif
+
     /* Set credentials */
     pamerr = pam_setcred (pamh, PAM_ESTABLISH_CRED);
     if (pamerr != PAM_SUCCESS) {
@@ -1254,7 +1352,8 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	    goto setup_pamerr;
     }
 
-    opened_session = TRUE;
+    credentials_set = TRUE;
+    opened_session  = TRUE;
 
     /* Register the session */
     pamerr = pam_open_session (pamh, 0);
@@ -1280,12 +1379,12 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
     extra_standalone_message = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->attached)
-		(void) di_devperm_login("/dev/console", pwent->pw_uid,
-		    pwent->pw_gid, NULL);
+    if (d->attached)
+	(void) di_devperm_login ("/dev/console", pwent->pw_uid,
+	    pwent->pw_gid, NULL);
 #endif	/* HAVE_LOGINDEVPERM */
 #ifdef  HAVE_ADT
-    audit_success_login(pw_change, pwent);
+    audit_success_login (pw_change, pwent);
 #endif	/* HAVE_ADT */
     
     return TRUE;
@@ -1293,7 +1392,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
  setup_pamerr:
 
 #ifdef  HAVE_ADT
-    audit_fail_login(d, pw_change, pwent, pamerr);
+    audit_fail_login (d, pw_change, pwent, pamerr);
 #endif	/* HAVE_ADT */
 
     did_setcred = FALSE;
@@ -1308,6 +1407,9 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	    gdm_sigchld_block_pop ();
 	    gdm_sigterm_block_pop ();
 
+	    /* Throw away the credentials */
+	    if (credentials_set)
+		    pam_setcred (tmp_pamh, PAM_DELETE_CRED);
 	    pam_end (tmp_pamh, pamerr);
     }
     pamh = NULL;
@@ -1364,7 +1466,7 @@ gdm_verify_cleanup (GdmDisplay *d)
 		 * If logged in, audit logout before cleaning up
 		 */
 		if (old_opened_session && old_did_setcred) {
-			audit_logout();
+			audit_logout ();
 		}
 #endif	/* HAVE_ADT */
 		/* Close the users session */
@@ -1382,13 +1484,12 @@ gdm_verify_cleanup (GdmDisplay *d)
 		pam_end (tmp_pamh, pamerr);
 
 #ifdef  HAVE_LOGINDEVPERM
-        if (old_opened_session && old_did_setcred && d->attached)
-        {
-            (void) di_devperm_logout("/dev/console");
-            /* give it back to gdm user */
-            (void) di_devperm_login("/dev/console", GdmUserId, GdmGroupId,
-                NULL);
-        }
+		if (old_opened_session && old_did_setcred && d->attached) {
+			(void) di_devperm_logout ("/dev/console");
+			/* give it back to gdm user */
+			(void) di_devperm_login ("/dev/console", GdmUserId,
+				GdmGroupId, NULL);
+		}
 #endif  /* HAVE_LOGINDEVPERM */
 
 		/* Workaround to avoid gdm messages being logged as PAM_pwdb */
