@@ -92,7 +92,7 @@ static void gdm_chooser_abort (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 static void gdm_chooser_warn (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
 /* Exported for glade */
-void gdm_chooser_cancel (/*void*/);
+void gdm_chooser_cancel (void);
 void gdm_chooser_add_host (void);
 void gdm_chooser_add_entry_changed (void);
 void gdm_chooser_manage (GtkButton *button, gpointer data);
@@ -174,6 +174,11 @@ static gchar *GdmMulticastAddr;
 /* */  gchar *GdmSoundProgram;
 /* */  gboolean GdmSoundOnLoginReady;
 /* */  gchar *GdmSoundOnLoginReadyFile;
+/* */  void
+/* */  gdm_kill_thingies (void)
+/* */  {
+/* */	  ; /* nothing */
+/* */  }
 
 
 enum {
@@ -505,7 +510,7 @@ gdm_chooser_decode_packet (GIOChannel   *source,
 	    if (! XdmcpReadARRAY8 (&buf, &stat))
 		    goto done;
 
-	    status = g_strndup ((char *) stat.data, MIN (stat.length, 256));
+	    status = g_strndup (stat.data, MIN (stat.length, 256));
     } else if (header.opcode == UNWILLING) {
 	    /* immaterial, will not be shown */
 	    status = NULL;
@@ -633,22 +638,26 @@ gdm_chooser_decode_packet (GIOChannel   *source,
 
     if (host_not_willing) {
 	    GtkWidget *dialog;
+	    gchar *msg;
 
 	    if (add_check_handler > 0)
 		    g_source_remove (add_check_handler);
 	    add_check_handler = 0;
 
+	    msg = g_strdup_printf (_("The host \"%s\" is not willing "
+	                             "to support a login session right now.  "
+	                             "Please try again later."),
+	                           added_host);
+
 	    dialog = ve_hig_dialog_new
 		    (GTK_WINDOW (chooser) /* parent */,
 		     GTK_DIALOG_MODAL /* flags */,
 		     GTK_MESSAGE_ERROR,
 		     GTK_BUTTONS_OK,
-		     FALSE /* markup */,
 		     _("Cannot connect to remote server"),
-		     _("The host \"%s\" is not willing "
-		       "to support a login session right now.  "
-		       "Please try again later."),
-		     added_host);
+		     msg);
+
+	    g_free (msg);
 
 	    if (RUNNING_UNDER_GDM)
 		    gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -1262,21 +1271,25 @@ add_check (gpointer data)
 
 	if (check) {
 		GtkWidget *dialog;
+		gchar *msg;
+		
+		msg = g_strdup_printf (_("Did not receive any response from host \"%s\" "
+		                         "in %d seconds.  Perhaps the host is not "
+		                         "turned on, or is not willing to support a "
+		                         "login session right now.  Please try again "
+		                         "later."),
+		                       added_host,
+		                       ADD_TIMEOUT / 1000);
 
 		dialog = ve_hig_dialog_new
 			(GTK_WINDOW (chooser) /* parent */,
 			 GTK_DIALOG_MODAL /* flags */,
 			 GTK_MESSAGE_ERROR,
 			 GTK_BUTTONS_OK,
-			 FALSE /* markup */,
 			 _("Did not receive response from server"),
-			 _("Did not receive any response from host \"%s\" "
-			   "in %d seconds.  Perhaps the host is not "
-			   "turned on, or is not willing to support a "
-			   "login session right now.  Please try again "
-			   "later."),
-			 added_host,
-			 ADD_TIMEOUT / 1000);
+			 msg);
+
+		g_free (msg);
 
 		if (RUNNING_UNDER_GDM)
 			gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -1319,11 +1332,13 @@ gdm_chooser_add_host (void)
 	qa6 = g_new0 (struct sockaddr_in6, 1);
 	qa6->sin6_family = AF_INET6;
 
-	if (have_ipv6 && strlen (name) == 32 && from_hex (name, (char *) &qa6->sin6_addr, strlen (name)) == 0) ;
+	if (have_ipv6 && strlen (name) == 32 && 
+	    from_hex (name, (char *) &qa6->sin6_addr, strlen (name)) == 0) ;
 
 	else
 #endif
-	if (strlen (name) == 8 && from_hex (name, (char *) &qa->sin_addr, strlen (name)) == 0) {
+	if (strlen (name) == 8 &&
+	    from_hex (name, (char *) &qa->sin_addr, strlen (name)) == 0) {
 #ifdef ENABLE_IPV6
 		if (have_ipv6) {
 			char tmpaddr[30];
@@ -1358,38 +1373,49 @@ gdm_chooser_add_host (void)
 			if (have_ipv6) {
 				char tmpaddr [30];
 
-				sprintf (tmpaddr, "::ffff:%s", inet_ntoa(((struct sockaddr_in *)result->ai_addr)->sin_addr));
+				sprintf (tmpaddr, "::ffff:%s",
+				  inet_ntoa(((struct sockaddr_in *)result->ai_addr)->sin_addr));
 				inet_pton (AF_INET6, tmpaddr, &qa6->sin6_addr);
 			}
 		}
 	}
 	else
 #endif
-	if ((hostent = gethostbyname (name)) != NULL && hostent->h_addrtype == AF_INET && hostent->h_length == 4) {
-					memmove (&qa->sin_addr, hostent->h_addr, 4);
+	if ((hostent = gethostbyname (name)) != NULL &&
+	     hostent->h_addrtype == AF_INET && hostent->h_length == 4) {
+		memmove (&qa->sin_addr, hostent->h_addr, 4);
 	} else {
-					GtkWidget *dialog;
+		GtkWidget *dialog;
+		gchar *msg;
 
-					dialog = ve_hig_dialog_new
-					(GTK_WINDOW (chooser) /* parent */,
-					 GTK_DIALOG_MODAL /* flags */,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_OK,
-					 FALSE,
-					 _("Cannot find host"),
-					 _("Cannot find the host \"%s\". "
-					   "Perhaps you have mistyped it."),
-					 name);
+		msg = g_strdup_printf (_("Cannot find the host \"%s\". "
+		                         "Perhaps you have mistyped it."),
+		                       name);
 
-					if (RUNNING_UNDER_GDM)
-						gdm_wm_center_window (GTK_WINDOW (dialog));
+		dialog = ve_hig_dialog_new
+		(GTK_WINDOW (chooser) /* parent */,
+		 GTK_DIALOG_MODAL /* flags */,
+		 GTK_MESSAGE_ERROR,
+		 GTK_BUTTONS_OK,
+		 _("Cannot find host"),
+		 msg);
+		 
+		g_free (msg);
+
+		if (RUNNING_UNDER_GDM)
+			gdm_wm_center_window (GTK_WINDOW (dialog));
+
+		gdm_wm_no_login_focus_push ();
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+		gdm_wm_no_login_focus_pop ();
+		g_free (qa);
+#ifdef ENABLE_IPV6
+		g_free (qa6);
+#endif
+		return; /* not a valid address */
+	}
 
-					gdm_wm_no_login_focus_push ();
-					gtk_dialog_run (GTK_DIALOG (dialog));
-					gtk_widget_destroy (dialog);
-					gdm_wm_no_login_focus_pop ();
-					return; /* not a valid address */
-				}
 #ifdef ENABLE_IPV6
 	if (have_ipv6) {
 		memset (&sock6, 0, sizeof (struct sockaddr_in6));
@@ -1458,6 +1484,10 @@ gdm_chooser_add_host (void)
 
 		/* empty the text entry to indicate success */
 		gtk_entry_set_text (GTK_ENTRY (add_entry), "");
+		g_free (qa);
+#ifdef ENABLE_IPV6
+		g_free (qa6);
+#endif
 		return;
 	}
 #ifdef ENABLE_IPV6
@@ -1488,11 +1518,9 @@ gdm_chooser_add_host (void)
 	/* empty the text entry to indicate success */
 	gtk_entry_set_text (GTK_ENTRY (add_entry), "");
 
-	if (have_ipv6)
-		g_free (qa);
+	g_free (qa);
 #ifdef ENABLE_IPV6
-	else
-		g_free (qa6);
+	g_free (qa6);
 #endif
 }
 
@@ -1506,7 +1534,7 @@ gdm_chooser_add_entry_changed (void)
 }
 
 void
-gdm_chooser_cancel (/*void*/)
+gdm_chooser_cancel (void)
 {
     if (scan_time_handler > 0) {
 	    g_source_remove (scan_time_handler);
@@ -1551,6 +1579,7 @@ gdm_chooser_abort (const gchar *format, ...)
 
     syslog (LOG_ERR, "%s", s);
     closelog ();
+    g_free (s);
 
     exit (EXIT_FAILURE);
 }
@@ -1567,6 +1596,7 @@ gdm_chooser_warn (const gchar *format, ...)
 
     syslog (LOG_ERR, "%s", s);
     closelog ();
+    g_free (s);
 }
 
 
@@ -1918,7 +1948,7 @@ gdm_chooser_signals_init (void)
     sigemptyset(&hup.sa_mask);
     sigaddset (&hup.sa_mask, SIGCHLD);
 
-    term.sa_handler = gdm_chooser_cancel;
+    term.sa_handler = (void *) gdm_chooser_cancel;
     term.sa_flags = 0;
     sigemptyset (&term.sa_mask);
 
@@ -1989,7 +2019,6 @@ main (int argc, char *argv[])
     int nextopt;
     const char *gdm_version;
     int i;
-    guint sid;
 
     stored_argv = g_new0 (char *, argc + 1);
     for (i = 0; i < argc; i++)
@@ -2050,23 +2079,26 @@ main (int argc, char *argv[])
 	gdm_version != NULL &&
 	strcmp (gdm_version, VERSION) != 0) {
 	    GtkWidget *dialog;
+	    gchar *msg;
 
 	    gdm_wm_init (0);
 
 	    gdm_wm_focus_new_windows (TRUE);
 
+	    msg = g_strdup_printf (_("The chooser version (%s) does not match the daemon "
+	                             "version (%s).  "
+	                             "You have probably just upgraded gdm.  "
+	                             "Please restart the gdm daemon or the computer."),
+	                           VERSION, gdm_version);
+
 	    dialog = ve_hig_dialog_new (NULL /* parent */,
 					GTK_DIALOG_MODAL /* flags */,
 					GTK_MESSAGE_ERROR,
 					GTK_BUTTONS_OK,
-					FALSE /* markup */,
 					_("Cannot run chooser"),
-					_("The chooser version (%s) does not match the daemon "
-					  "version (%s).  "
-					  "You have probably just upgraded gdm.  "
-					  "Please restart the gdm daemon or the computer."),
-					VERSION, gdm_version);
-
+					msg);
+	    g_free (msg);
+	    
 	    gtk_widget_show_all (dialog);
 	    gdm_wm_center_window (GTK_WINDOW (dialog));
 
@@ -2093,7 +2125,7 @@ main (int argc, char *argv[])
     gdm_chooser_xdmcp_init (hosts);
     poptFreeContext (ctx);
 
-    sid = g_signal_lookup ("event",
+    guint sid = g_signal_lookup ("event",
 				 GTK_TYPE_WIDGET);
     g_signal_add_emission_hook (sid,
 				0 /* detail */,
