@@ -1,4 +1,5 @@
-/*
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
+ *
  *    GDMflexiserver - run a flexible server
  *    (c)2001 Queen of England
  *    
@@ -19,9 +20,6 @@
  */
 
 #include "config.h"
-#include <libgnome/libgnome.h>
-#include <libgnomeui/libgnomeui.h>
-#include <gdk/gdkx.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -35,6 +33,12 @@
 #include <errno.h>
 #include <pwd.h>
 
+#include <glib/gi18n.h>
+#include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libgnome/libgnome.h> /* for gnome_config */
+#include <libgnomeui/libgnomeui.h> /* for gnome_program */
+
 #include <viciousui.h>
 
 #include "gdm.h"
@@ -51,6 +55,7 @@ static const char *send_command = NULL;
 static const char *server = NULL;
 static const char *chosen_server = NULL;
 static gboolean debug = FALSE;
+static gboolean startnew = FALSE;
 static char *auth_cookie = NULL;
 
 static int
@@ -96,9 +101,8 @@ change_vt (int vt)
 			 GTK_DIALOG_MODAL /* flags */,
 			 GTK_MESSAGE_ERROR,
 			 GTK_BUTTONS_OK,
-			 FALSE /* markup */,
 			 _("Cannot change display"),
-			 "%s", message);
+			 message);
 
 		gtk_widget_show_all (dialog);
 		gtk_dialog_run (GTK_DIALOG (dialog));
@@ -295,6 +299,60 @@ row_activated (GtkTreeView *view, GtkTreePath *path, GtkTreeViewColumn *col, gpo
 	gtk_dialog_response (GTK_DIALOG (dialog), RESPONSE_OPEN_EXISTING_DISPLAY);
 }
 
+static gboolean
+is_program_in_path (const char *program)
+{
+	char *tmp = g_find_program_in_path (program);
+	if (tmp != NULL) {
+		g_free (tmp);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
+static void
+maybe_lock_screen (void)
+{
+	gboolean   use_gscreensaver = FALSE;
+	GError    *error            = NULL;
+	char      *command;
+	GdkScreen *screen;
+
+	if (is_program_in_path ("gnome-screensaver-command"))
+		use_gscreensaver = TRUE;
+	else if (! is_program_in_path ("xscreensaver-command"))
+		return;
+
+	if (use_gscreensaver) {
+		command = g_strdup ("gnome-screensaver-command --lock");
+	} else {
+		command = g_strdup ("xscreensaver-command -lock");
+	}
+
+	screen = gdk_screen_get_default ();
+
+	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+		g_warning ("Cannot lock screen: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (command);
+
+	if (use_gscreensaver) {
+		command = g_strdup ("gnome-screensaver-command --throttle");
+	} else {
+		command = g_strdup ("xscreensaver-command -throttle");
+	}
+
+	if (! gdk_spawn_command_line_on_screen (screen, command, &error)) {
+		g_warning ("Cannot disable screensaver engines: %s", error->message);
+		g_error_free (error);
+	}
+
+	g_free (command);
+}
+
 static void
 run_logged_in_dialogue (char **vec)
 {
@@ -306,6 +364,12 @@ run_logged_in_dialogue (char **vec)
 	GtkWidget *treeview;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
+	gint response;
+
+	if (startnew == TRUE) {
+		/* Just return if the user doesn't want to see the dialog */
+		return;
+	} 
 
 	dialog = gtk_dialog_new_with_buttons (_("Open Displays"),
 					      NULL /* parent */,
@@ -321,7 +385,7 @@ run_logged_in_dialogue (char **vec)
 	vbox = GTK_DIALOG (dialog)->vbox;
 
 	w = gtk_label_new (_("There are some displays already open.  You can select "
-			     "one from the list below or open a new one."));
+		     "one from the list below or open a new one."));
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
@@ -355,20 +419,22 @@ run_logged_in_dialogue (char **vec)
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
 
-        g_signal_connect (selection, "changed",
+	g_signal_connect (selection, "changed",
 			  G_CALLBACK (selection_changed),
 			  dialog);
 
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog),
-					   RESPONSE_OPEN_EXISTING_DISPLAY,
-					   FALSE);
+				   RESPONSE_OPEN_EXISTING_DISPLAY,
+				   FALSE);
 
 	gtk_widget_show_all (dialog);
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 run_again:
-	switch (gtk_dialog_run (GTK_DIALOG (dialog))) {
+	switch (response) {
 	case RESPONSE_OPEN_NEW_DISPLAY:
 		gtk_widget_destroy (dialog);
+
 		/* just continue what you are doing */
 		return;
 
@@ -385,18 +451,13 @@ run_again:
 			/* we switched to a different screen as a result of this,
 			 * lock the current screen */
 			if ( ! no_lock && vt != get_cur_vt () && vt >= 0) {
-				char *argv[3] = {"xscreensaver-command", "-lock", NULL};
-				if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-					g_warning (_("Can't lock screen"));
-				argv[1] = "-throttle";
-				if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-					g_warning (_("Can't disable xscreensaver display hacks"));
+				maybe_lock_screen ();
 			}
 
 			change_vt (vt);
 
-			/* FIXME: wait + disturb the pointer (need SUP?), perhaps
-			 * part of the sup command to CHVT ?? */
+			/* FIXME: wait + disturb the pointer (need SUP?), 
+			 * perhaps part of the sup command to CHVT ?? */
 
 			exit (0);
 		} else {
@@ -486,7 +547,7 @@ read_servers (gchar *config_file)
 			char *section;
 			GdmXServer *svr;
 
-			section = g_strdup_printf ("=%s=/", config_file);
+			section = g_strdup_printf ("=%s=/%s/", config_file, k);
 			gnome_config_push_prefix (section);
 
 			if ( ! gnome_config_get_bool
@@ -635,6 +696,7 @@ struct poptOption options [] = {
 	{ "no-lock", 'l', POPT_ARG_NONE, &no_lock, 0, N_("Do not lock current screen"), NULL },
 	{ "debug", 'd', POPT_ARG_NONE, &debug, 0, N_("Debugging output"), NULL },
 	{ "authenticate", 'a', POPT_ARG_NONE, &authenticate, 0, N_("Authenticate before running --command"), NULL },
+	{ "startnew", 's', POPT_ARG_NONE, &startnew, 0, N_("Start new flexible session; do not show popup"), NULL },
 	{ "monte-carlo-pi", 0, POPT_ARG_NONE, &monte_carlo_pi, 0, NULL, NULL },
 	POPT_AUTOHELP
 	{ NULL, 0, 0, NULL, 0}
@@ -704,10 +766,8 @@ main (int argc, char *argv[])
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("Cannot communicate with GDM "
 				   "(The GNOME Display Manager)"),
-				 "%s",
 				 _("Perhaps you have an old version "
 				   "of GDM running."));
 			gtk_widget_show_all (dialog);
@@ -733,11 +793,9 @@ main (int argc, char *argv[])
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("You do not seem to have the "
 				   "authentication needed for this "
 				   "operation"),
-				 "%s",
 				 _("Perhaps your .Xauthority "
 				   "file is not set up correctly."));
 
@@ -762,10 +820,8 @@ main (int argc, char *argv[])
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 _("You do not seem to be logged in on the "
 				   "console"),
-				 "%s",
 				 _("Starting a new login only "
 				   "works correctly on the console."));
 			gtk_dialog_set_has_separator (GTK_DIALOG (dialog),
@@ -798,12 +854,7 @@ main (int argc, char *argv[])
 		/* if we switched to a different screen as a result of this,
 		 * lock the current screen */
 		if ( ! no_lock && ! use_xnest) {
-			char *argv[3] = {"xscreensaver-command", "-lock", NULL};
-			if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-				g_warning (_("Can't lock screen"));
-			argv[1] = "-throttle";
-			if (gnome_execute_async (g_get_home_dir (), 2, argv) < 0)
-				g_warning (_("Can't disable xscreensaver display hacks"));
+			maybe_lock_screen ();
 		}
 
 		/* all fine and dandy */
@@ -817,9 +868,8 @@ main (int argc, char *argv[])
 		 GTK_DIALOG_MODAL /* flags */,
 		 GTK_MESSAGE_ERROR,
 		 GTK_BUTTONS_OK,
-		 FALSE /* markup */,
 		 _("Cannot start new display"),
-		 "%s", message);
+		 message);
 
 	gtk_widget_show_all (dialog);
 	gtk_dialog_run (GTK_DIALOG (dialog));
