@@ -159,20 +159,23 @@ set_face_from_filename (const char *filename)
 	if (gdk_pixbuf_save (pixbuf, photofile, "png", NULL, NULL) != TRUE) {
 		GtkWidget *d;
 		char	  *tmp = g_filename_to_utf8 (photofile, -1, NULL, NULL, NULL);
+		char      *msg;
+		
+		msg = g_strdup_printf (_("File %s cannot be opened for "
+					 "writing."), tmp);
 
 		d = ve_hig_dialog_new (NULL /* parent */,
 				       GTK_DIALOG_MODAL /* flags */,
 				       GTK_MESSAGE_ERROR,
 				       GTK_BUTTONS_OK,
-				       FALSE /* markup */,
 				       _("Cannot open file"),
-				       _("File %s cannot be opened for "
-					 "writing\n"), tmp);
+				       msg);
 
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (d);
 
 		g_free (tmp);
+		g_free (msg);
 	} else {
 		/* Change to g_chmod after glib 2.8 release */
 		chmod (photofile, 0644);
@@ -198,7 +201,7 @@ install_response (GtkWidget *file_dialog,
 	gtk_widget_destroy (file_dialog);
 }
 
-void
+static void
 add_preview_widget (GtkWidget *widget)
 {
 	GtkWidget *vbox;
@@ -236,6 +239,7 @@ browse_button_cb (GtkWidget *widget, gpointer data)
 						   GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 						   GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
 						   NULL);
+
 	gtk_file_chooser_set_show_hidden (GTK_FILE_CHOOSER (file_dialog), TRUE);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_dialog), TRUE);
 
@@ -373,8 +377,10 @@ fill_model (GtkTreeModel *model)
 
 		path = g_build_filename (facedir, filename, NULL);
 		pixbuf = gdk_pixbuf_new_from_file (path, NULL);
-		if (! pixbuf)
+		if (! pixbuf) {
+			g_free (path);
 			continue;
+		}
 
 		gtk_list_store_prepend (store, &iter);
 
