@@ -257,9 +257,9 @@ greeter_parse_config (void)
     GdmDefaultWelcomeBacktest = ve_config_get_bool (config, greeter_DefaultWelcomeBacktest_key);
     if (GdmDefaultWelcomeBacktest == FALSE) {
             if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_WELCOME_MSG) == 0)
-                GdmDefaultWelcome = TRUE;
+                GdmDefaultWelcome == TRUE;
             else if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_REMOTEWELCOME_MSG) == 0)
-                GdmDefaultWelcome = TRUE;
+                GdmDefaultWelcome == TRUE;
             else
                 GdmDefaultWelcome = FALSE;
     }
@@ -384,6 +384,7 @@ greeter_ctrl_handler (GIOChannel *source,
 	buf[len-1] = '\0';
 	
 	greeter_item_pam_set_user (buf);
+	greeter_item_ulist_enable ();
 	printf ("%c\n", STX);
 	fflush (stdout);
 	break;
@@ -450,9 +451,8 @@ greeter_ctrl_handler (GIOChannel *source,
 				 GTK_DIALOG_MODAL /* flags */,
 				 GTK_MESSAGE_ERROR,
 				 GTK_BUTTONS_OK,
-				 FALSE /* markup */,
 				 tmp,
-				 /* avoid warning */ "%s", "");
+				 "");
 	g_free (tmp);
 
 	gdm_wm_center_window (GTK_WINDOW (dlg));
@@ -540,8 +540,6 @@ greeter_ctrl_handler (GIOChannel *source,
 	printf ("%c\n", STX);
 	fflush (stdout);
 	greeter_ignore_buttons (FALSE);
-	greeter_item_ulist_enable ();
-
 	break;
 
     case GDM_QUIT:
@@ -557,12 +555,11 @@ greeter_ctrl_handler (GIOChannel *source,
 					 GTK_DIALOG_MODAL /* flags */,
 					 GTK_MESSAGE_INFO,
 					 GTK_BUTTONS_OK,
-					 FALSE /* markup */,
 					 /* translators:  This is a nice and evil eggie text, translate
 					  * to your favourite currency */
 					 _("Please insert 25 cents "
 					   "to log in."),
-					 /* avoid warning */ "%s", "");
+					 "");
 		gdm_wm_center_window (GTK_WINDOW (dlg));
 
 		gdm_wm_no_login_focus_push ();
@@ -792,21 +789,24 @@ verify_gdm_version (void)
       (g_getenv ("GDM_IS_LOCAL") != NULL))
     {
       GtkWidget *dialog;
+      gchar *msg;
     
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
+      msg =  g_strdup_printf (_("The greeter version (%s) does not match the daemon "
+                                "version.  "
+                                "You have probably just upgraded gdm.  "
+                                "Please restart the gdm daemon or the computer."),
+                              VERSION);
+    
       dialog = ve_hig_dialog_new (NULL /* parent */,
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
-				  _("The greeter version (%s) does not match the daemon "
-				    "version.\n"
-				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or the computer."),
-				  VERSION);
+				  msg);
+      g_free (msg);
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -824,21 +824,25 @@ verify_gdm_version (void)
       gdm_version == NULL)
     {
       GtkWidget *dialog;
+      gchar *msg;
     
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
+      msg = g_strdup_printf (_("The greeter version (%s) does not match the daemon "
+                               "version. "
+                               "You have probably just upgraded gdm. "
+                               "Please restart the gdm daemon or the computer."),
+                             VERSION);
+    
       dialog = ve_hig_dialog_new (NULL /* parent */,
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_NONE,
-				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
-				  _("The greeter version (%s) does not match the daemon "
-				    "version.\n"
-				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or the computer."),
-				  VERSION);
+				  msg);
+      g_free (msg);
+			  
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Reboot"),
 			      RESPONSE_REBOOT,
@@ -869,21 +873,25 @@ verify_gdm_version (void)
 	strcmp (gdm_version, VERSION) != 0)))
     {
       GtkWidget *dialog;
+      gchar *msg;
     
       gdm_wm_init (0);
       gdm_wm_focus_new_windows (TRUE);
     
+      msg = g_strdup_printf (_("The greeter version (%s) does not match the daemon "
+                               "version (%s).  "
+                               "You have probably just upgraded gdm.  "
+                               "Please restart the gdm daemon or the computer."),
+                             VERSION, gdm_version);
+    
       dialog = ve_hig_dialog_new (NULL /* parent */,
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_WARNING,
 				  GTK_BUTTONS_NONE,
-				  FALSE /* markup */,
 				  _("Cannot start the greeter"),
-				  _("The greeter version (%s) does not match the daemon "
-				    "version (%s).\n"
-				    "You have probably just upgraded gdm.\n"
-				    "Please restart the gdm daemon or the computer."),
-				  VERSION, gdm_version);
+				  msg);
+      g_free (msg);
+
       gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 			      _("Restart"),
 			      RESPONSE_RESTART,
@@ -1007,9 +1015,9 @@ greeter_reread_config (int sig, gpointer data)
                  */
                 if (GdmDefaultWelcomeBacktest == FALSE) {
                         if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_WELCOME_MSG) == 0)
-                                GdmDefaultWelcome = TRUE;
+                                GdmDefaultWelcome == TRUE;
                         else if (strcmp (ve_sure_string (GdmWelcome), GDM_DEFAULT_REMOTEWELCOME_MSG) == 0)
-                                GdmDefaultWelcome = TRUE;
+                                GdmDefaultWelcome == TRUE;
                         else
                                 GdmDefaultWelcome = FALSE;
                 }
@@ -1122,6 +1130,7 @@ get_theme_file (const char *in, char **theme_dir)
     }
 
   config = ve_config_new (info);
+  g_free (info);
 
   s = ve_config_get_translated_string (config, "GdmGreeterTheme/Greeter");
   if (s == NULL || s[0] == '\0')
@@ -1159,6 +1168,14 @@ reap_flexiserver (gpointer data)
 	return TRUE;
 }
 
+
+static void
+gdm_kill_thingies (void)
+{
+	/* Empty kill thingies */
+	return;
+}
+
 static gboolean
 gdm_event (GSignalInvocationHint *ihint,
            guint                n_param_values,
@@ -1372,9 +1389,8 @@ main (int argc, char *argv[])
 				    GTK_DIALOG_MODAL /* flags */,
 				    GTK_MESSAGE_ERROR,
 				    GTK_BUTTONS_OK,
-				    FALSE /* markup */,
 				    s,
-				    "%s", (error && error->message) ? error->message : "");
+				    (error && error->message) ? error->message : "");
 	g_free (s);
     
         gtk_widget_show_all (dialog);
@@ -1418,9 +1434,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("The greeter theme is corrupt"),
-				  "%s",
 				  _("The theme does not contain "
 				    "definition for the username/password "
 				    "entry element."));
@@ -1454,13 +1468,12 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("There was an error loading the "
 				    "theme, and the default theme "
 				    "could not be loaded. "
 				    "Attempting to start the "
 				    "standard greeter"),
-				  /* avoid warning */ "%s", "");
+				  "");
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -1477,12 +1490,11 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("The GTK+ greeter could not be started.  "
 				    "This display will abort and you may "
 				    "have to login another way and fix the "
 				    "installation of gdm"),
-				  /* avoid warning */ "%s", "");
+				  "");
     
       gtk_widget_show_all (dialog);
       gdm_wm_center_window (GTK_WINDOW (dialog));
@@ -1505,9 +1517,6 @@ main (int argc, char *argv[])
   gtk_window_move (GTK_WINDOW (window), gdm_wm_screen.x, gdm_wm_screen.y);
   gtk_widget_show_now (window);
 
-  greeter_item_ulist_unset_selected_user ();
-  greeter_item_ulist_enable ();
-
   /* can it ever happen that it'd be NULL here ??? */
   if G_UNLIKELY (window->window != NULL)
     {
@@ -1529,9 +1538,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("Session directory is missing"),
-				  "%s",
 				  _("Your session directory is missing or empty!  "
 				    "There are two available sessions you can use, but "
 				    "you should log in and correct the gdm configuration."));
@@ -1556,9 +1563,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("Configuration is not correct"),
-				  "%s",
 				  _("The configuration file contains an invalid command "
 				    "line for the login dialog, so running the "
 				    "default command.  Please fix your configuration."));
@@ -1585,9 +1590,7 @@ main (int argc, char *argv[])
 				  GTK_DIALOG_MODAL /* flags */,
 				  GTK_MESSAGE_ERROR,
 				  GTK_BUTTONS_OK,
-				  FALSE /* markup */,
 				  _("No configuration was found"),
-				  "%s",
 				  _("The configuration was not found.  GDM is using "
 				    "defaults to run this session.  You should log in "
 				    "and create a configuration file with the GDM "
